
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif
/*
#ifdef EXPORT_LIB
#define WINDLL __declspec(dllexport)
#else
#define WINDLL __declspec(dllimport)
#endif
*/
#define WINDLL
#define MSG_USER_PAINT 0x0410

//-----------------------------------------------------------------------------
// Include global files
//-----------------------------------------------------------------------------
#include <windows.h>
#include <ddraw.h>
#include <winuserm.h>
#include <sipapi.h>
#include <shellapi.h>
#include <string>
#include <sstream>

#include <vector>
#include <aygshell.h>
#include <imaging.h>

using namespace std;

class MouseCtrl;
class EventHandler;
class DirectDrawWarp;
class MessageBoxSense;
class Section;

#define MANILA_SDK_VERSION 1.3
#define DRAWING_GDI
//#define DRAWING_DDRAW

#ifdef DRAWING_GDI
#define RGB555(r,g,b) (((0x001F&(WORD)r) << 10) |((0x1F&(WORD)g) << 5) |((0x1F&(WORD)b)))// RGB565 : RRRR RGGG GGGB BBBB (31,63,31)
#define RGB16(r,g,b) (((0x000000FF&(DWORD)r) << 19) |((0x000000FF&(DWORD)g) << 10) |((0x000000FF&(DWORD)b)<<3))// RGB32 (false 16Bit)
#else
#define RGB16(r,g,b) (((WORD)r << 11) |(0x3F&(WORD)g) << 5) |((0x1F&(WORD)b))// RGB565 : RRRR RGGG GGGB BBBB (31,63,31)
//#define RGB16(r,g,b) (((WORD)r << 11) |(((WORD)g) << 5) |(WORD)b)// RGB565 : RRRR RGGG GGGB BBBB (31,63,31)
#endif


//Directdraw caching...
typedef struct DIRECTDRAW_GDI
{
	IDirectDrawSurface* directDrawSurf;
	HDC deviceInterface;
	HBITMAP bitmap;
	BYTE nbBPP;
	int width,height;
	DIRECTDRAW_GDI(){directDrawSurf=NULL;deviceInterface=0;bitmap=0;nbBPP=0;};
}DIRECTDRAW_GDI;
typedef struct GDI_OVERLAY
{
	IDirectDrawSurface* directDrawSurf;
	HDC deviceInterface;
	HBITMAP bitmap;
	int width,height;
	HWND windowOverlay;
	RECT positionOut;
	bool visible;
	GDI_OVERLAY(){directDrawSurf=NULL;deviceInterface=0;bitmap=0;visible=false;};
}GDI_OVERLAY;

//-----------------------------------------------------------------------------
// GlobalInformation header
//-----------------------------------------------------------------------------

class WINDLL GlobalInformations
{
public:
	GlobalInformations(wstring name);
	~GlobalInformations(void);
	static void loadResources(wstring myDirectory);
	static wstring *bg_common_titlebar;
	static wstring *bg_common_divider_section;
	static wstring *btn_common_msgbox_large;
	static wstring *btn_common_msgbox_small;
	static wstring *btn_common_normal;
	static wstring *btn_switch_thirdState;
	static wstring *btn_checkbox_thirdState;
	static wstring *btn_common_switch;
	static wstring *btn_common_checkbox;
	static wstring *btn_common_slider;
	static wstring *btn_slider;
	static wstring *btn_common_input;
	static wstring *btn_common_listitem;
	static wstring *pivot_leftarrow;
	static wstring *pivot_rightarrow;
	static wstring *btn_common_combo_down;
	static wstring *btn_common_listitem_bottom;
	static wstring *btn_common_listitem_middle;
	static wstring *bg_common_dotline;
	static wstring *bg_msg_box;
	static wstring *bg_dark;

	static DirectDrawWarp* window;
	static HINSTANCE m_hInstance;
	static HWND   m_hWnd;
	static LPDIRECTDRAW *ldDDraw;
	static DIRECTDRAW_GDI *primaryBuffer;		// Primary Surface.
	static DIRECTDRAW_GDI *backBuffer;
	static DIRECTDRAW_GDI *background;			//to erase background
	static bool isActiv;
	static bool mustRender;
	static bool mustRedraw;
	static HANDLE waitForMSG;
	static double percentOfVGA;
	static bool portrait;
	static CRITICAL_SECTION criticPaint;
	static CRITICAL_SECTION criticAddBufPaint;
	static wstring userDirectory;


	static HFONT hFontTitle;


	//surface size :
	static RECT rs;
	static RECT rcUpdateRegion;
	static RECT rcVisibleDesktopTOP;
	static RECT rcVisibleDesktopBOTTOM;
};


//-----------------------------------------------------------------------------
// LocalisedString header
//-----------------------------------------------------------------------------

class WINDLL  LocalisedString{
public:
	//-----------------------------------------------------------------------------
	// Localized data
	//-----------------------------------------------------------------------------
	static TCHAR*	windowCreationError;
	static TCHAR*	LoadImageError;
	static TCHAR*	SizeImageError;
	static TCHAR*	AddElemError;
	static TCHAR*	ResourceError;
	static TCHAR*	szDDrawError;
	static TCHAR*	szDDrawFailedMsg;
	static TCHAR*	szSetCooperativeFailMsg;
	static TCHAR*	szNoOverlaysMsg;
	static TCHAR*	szCreateOverlayFailMsg;
	static TCHAR*	szLoadOverlayFailMsg;
	static TCHAR*	szShowOverlayFailMsg;
	static TCHAR*	szNoFlipSurfacesMsg;
	static TCHAR*	szEnumAttachedSurfacesFailMsg;
	static TCHAR*	szCreateSurfaceFailMsg;
	static TCHAR*	szSetTimerFailMsg;
	static TCHAR*	szGetCapsFailMsg;
	static TCHAR*	szNoBackBufferMsg;
	static TCHAR*	szSurfaceLost;
	static TCHAR*	szGetDC;
	static TCHAR*	szCreateInputText;
	static TCHAR*	szShowKbrd;
	static TCHAR*	szCreateMyItem;


	//print debug information when it fail :
	static void explainFail(LPCTSTR error,LPCTSTR title,int quit=1);
	//print debug information when DirectDraw fail :
	static void explainFailDirectDraw(int hRet,wstring info);
	//print debug information when Imaging fail :
	static void explainFailIImage(HRESULT hRet,wstring info);
	//print debug information when Win API fail :
	static void explainFailWinAPI(wstring info);

};

#define TYPE_CHILD_SECTION	1
#define TYPE_EDIT_CTRL		2
#define TYPE_ITEM			3
#define TYPE_LABEL			4
#define TYPE_PANEL			5
#define TYPE_PICTURE		6
#define TYPE_SELECTOR		7
#define TYPE_SECTION		8
#define TYPE_SLIDER			9

//////////////////////////////////////////////////////////////////////////
//class WINDLL  Element : provide basic function...
//////////////////////////////////////////////////////////////////////////

class WINDLL  Element
{
	static vector<Element*> listElements;
protected:
	RECT position;//the position relative to other components
	RECT positionAbsolute;//the real position on screen (updated each frame)
	bool isActivated;//for button, ctrl, etc...
	MouseCtrl* mouseEvent;//can ask for changes
	int idElement;//id used by event handler
	int paddingSup;
	int typeElement;

	//load (or reload) surface
	virtual void loadSurf()=0;
public:
	static void reloadAllSurface();
	static EventHandler* gestEvent;
	Element* owner;//Element which contain this Element

	Element(int idElement,int &x,int &y,int typeElement,bool realPos=false);
	virtual ~Element(void);

	friend class MessageBoxSense;
	//forcing to redraw
	virtual void setMustRedraw();
	
//-----------------------------------------------------------------------------
// Virtual pure. Must be overriden !
//-----------------------------------------------------------------------------

	//to paint component with correct offset :
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y)=0;

	//find element which is clicked :
	virtual Element* isClicked(int x,int y)=0;


//-----------------------------------------------------------------------------
// Optionnal behaviour :
//-----------------------------------------------------------------------------
	//execute action (Button, text...):
	virtual bool run();//return true if Element take control of events...
	//add componnent (list...)
	virtual void add(Element* e,int minHeight=-1,int pos=-1,bool mixWithOther=true);
	//Capture the mouse to interact with user
	virtual int mouseGesture(){return 0;};

	
//-----------------------------------------------------------------------------
// Getter and setter
//-----------------------------------------------------------------------------
	static IImagingFactory *pImgFactory;
	static DIRECTDRAW_GDI * loadImageWithAlpha(wstring name,bool rescall);
	static DIRECTDRAW_GDI * loadImage(wstring *name,bool rescall=false,bool transparent=false);
	static DIRECTDRAW_GDI* CreateBitmapMask(DIRECTDRAW_GDI *hbmColour, COLORREF crTransparent);
	static IImage * loadIImage(wstring *name);
	static void resizeIImage(wstring *name);
	static RECT patchImage(DIRECTDRAW_GDI * bkg,wstring *name,int nbPatch,int posX,int posY);

	inline int getWidth(){return position.right-position.left;};
	inline int getHeight(){return position.bottom-position.top;};
	inline int getTop(){return position.top;};
	inline int getLeft(){return position.left;};
	inline int getBottom(){return position.bottom;};
	inline int getRight(){return position.right;};
	inline int getId(){return idElement;};
	inline int getType(){return typeElement;};
/*
	inline void setTop(int t){position.top=t>0?t:0;};
	inline void setBottom(int t){position.bottom=t<GetSystemMetrics(SM_CYSCREEN)?t:GetSystemMetrics(SM_CYSCREEN);};
	inline void setRight(int t){position.right=t<GetSystemMetrics(SM_CXSCREEN)?t:GetSystemMetrics(SM_CXSCREEN);};
	inline void setLeft(int t){position.left=t>0?t:0;};
	*/
	inline void setWidth(int t){position.right=position.left+t;};
	inline void setHeight(int t){position.bottom=position.top+t;};
	inline void setTop(int t){position.top=t;};
	inline void setBottom(int t){position.bottom=t;};
	inline void setRight(int t){position.right=t;};
	inline void setLeft(int t){position.left=t;};
	inline void recopiePositionAbsolute(){positionAbsolute=position;};

	static void redraw();
	
	virtual void unClick(){isActivated=false;};

	static Element* getElementByID(int id,int typeExpected=-1);
	virtual void sizeChanged(Element* src,int width,int height);
};


//-----------------------------------------------------------------------------
// DirectDrawWarp MyPictureBox
//-----------------------------------------------------------------------------


class WINDLL  MyPictureBox :
	public Element
{
	DIRECTDRAW_GDI *pImage;
	RECT clipping;
	int numImage;
	int nbMaxImage;
	int widthImg;
	//load (or reload) surface
	virtual void loadSurf();
public:
	//height and width is found using the picture...
	MyPictureBox(int idElement, int x,int y,wstring img,int nbImg=1,bool rescall=false);
	MyPictureBox(int idElement, int x,int y,DIRECTDRAW_GDI *i,int nbImg=1);
	~MyPictureBox(void);

	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	virtual Element* isClicked(int x,int y);

	void setNumImage(int i){numImage=i<nbMaxImage?i:0;};


	void changeImg(DIRECTDRAW_GDI *surf){pImage=surf;};
};

//-----------------------------------------------------------------------------
// MyLabel header
//-----------------------------------------------------------------------------

#define LABEL_STATE_NORMAL		0
#define LABEL_STATE_HOVER		1
#define LABEL_STATE_DSBL		2
#define LABEL_STATE_SELECTED	3

#define TEXT_NORMAL 0
#define TEXT_BIG 1
#define TEXT_ITALIC 2

class WINDLL  MyLabel :
	public Element
{
	HFONT myFont;

	DIRECTDRAW_GDI *render;
	DIRECTDRAW_GDI *renderMask;
	wstring myText;
	RECT clipping;
	RECT sizeSurface;
	DWORD previousColor;
	DWORD myColor;
	DWORD backColor;
	DDCOLORKEY ddck;

	bool mustErase;
	bool isVisible;
	int lastWidth;
	int paddingTop;
	int paddingLeft;

	MyPictureBox *backGrnd;
	void initComponnents(int type,DWORD color);
	//load (or reload) surface
	virtual void loadSurf();
public:
	static HFONT fontTitle;
	static HFONT fontNormal;
	static HFONT fontItalic;

	//not always need width and height...
	MyLabel(int idElement, int x,int y,int width,int height,wstring text,int fontType=0,bool centered=true,DWORD color=0);
	MyLabel(int idElement, int x,int y,wstring text,int fontType=0,DWORD color=0,bool realPos=false,DWORD backColor=1);
	MyLabel(int idElement, int x,int y,wstring text,DIRECTDRAW_GDI *bkgHover,int fontType=0,DWORD color=0);
	MyLabel(int idElement, int x,int y,wstring text,MyPictureBox* image,int paddingText,int fontType=0,DWORD color=0);
	~MyLabel(void);

	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	virtual Element* isClicked(int x,int y);

	virtual void paintColor(DIRECTDRAW_GDI* lpDDraw,int x,int y,COLORREF color,COLORREF background);

	void setText(wstring nT,bool redraw=true);
	wstring getText(){return myText;};
	int getState(){return state;};

	bool run();
	void unClick();

	bool isClickable;
	int state;
	void center();
	void centerV(int top=-1,int bottom=-1);
	void centerH(int left,int right);
	void setPaddingTop(int t){paddingTop=t;};
	void setPaddingLeft(int l){paddingLeft=l;};

	int getPaddingTop(){return paddingTop;};
	int getPaddingLeft(){return paddingLeft;};

	void setBkg(DIRECTDRAW_GDI *surf);
	void changeColor(DWORD color,bool foregroundColor=true);

	void changeVisibility(bool vis,bool eraseBckground=true);
};

//-----------------------------------------------------------------------------
// MouseCtrl header
//-----------------------------------------------------------------------------



#define NB_MOUSE_HISTORY 3
#define DBL_CLICK_LENGTH 400
#define VEL_INERTIE 100.0

class WINDLL  MouseCtrl
{
	//Mouse interface :
	int mouseOldXPos, mouseOldYPos;
	int mouseClickXPos, mouseClickYPos;
	int mouseNewXPos, mouseNewYPos;
	int lastMouseXPos, lastMouseYPos;
	bool hasInfoAboutClick;//to know if user get the information about click
	bool splClick;
	bool mouseDown;
	bool clickSelected;
	bool dblClick;
	bool scrolling;
	bool somethingAppend;
	bool slideLeft;

	DWORD lastTickMouse;
	DWORD clickEvent;
	DWORD startScrolling;
	DWORD lastScroll;
	double userXVel[NB_MOUSE_HISTORY], userYVel[NB_MOUSE_HISTORY];
	int idPos;

	static CRITICAL_SECTION mouseUpdate; // This is the critical section object -- once initialized, it cannot be moved in memory

	Element* gestMouse;


	//private constructor : Singleton
	MouseCtrl(void);
	static MouseCtrl* instance;
	bool cantSlideLeft;
public:
	~MouseCtrl(void);

	static MouseCtrl* getInstance();
	int updateElement();
	Element* giveControlBack(){
		if (gestMouse!=NULL){
			Element* out=gestMouse;
			gestMouse->unClick();
			return out;
		};
		return NULL;
	};


	//-----------------------------------------------------------------------------
	// Setter
	//-----------------------------------------------------------------------------

	void leftBtnDown(int x,int y);
	void leftBtnUp(int x,int y);
	void leftBtnMove(int x,int y);
	bool setMouseAnalyse(Element* e);
	Element* changeMouseAnalyse(Element* e);
	void stopScroll();
	bool pickChange(){bool out=somethingAppend||scrolling;somethingAppend=false;return out;};
	void setClickSelected(bool val){clickSelected=val;};



	//-----------------------------------------------------------------------------
	// Getter
	//-----------------------------------------------------------------------------
	int getMouseDY();
	int getMouseDX();
	double getMouseDXMean();
	bool isMouseDown();
	bool isSelected();
	bool isSlidingLeft();
	bool isClicked();
	bool isDblClicked();
	bool sthgHasChange();
	void getMousePosition(int &x,int &y);
};


//-----------------------------------------------------------------------------
// MySlider header
//-----------------------------------------------------------------------------

#define VIS_SLIDE_NORMAL	0
#define VIS_SLIDE_DISABLED	2

class MySlider :
	public Element
{
protected:

	static DIRECTDRAW_GDI *slideSurf;
	static DIRECTDRAW_GDI *btnSurf;

	RECT clipping;
	static int widthSlide;
	static int heightSlide;
	static int widthBtn;
	static int heightBtn;
	Section* container;

	int state;
	int position;
	int minVal,maxVal;
	bool sliderAnim;
	bool isSliding;
	void loadSurf();
public:
	//to create a common slider
	MySlider(int idElement, int x,int y,int width,bool visible,int minVal=0,int maxVal=100);
	//to create a slider which return to origin when released
	MySlider(int idElement, int x,int y,int width,bool visible, double lengthAnim);
	~MySlider(void);

	void updatePosition(int newPos=-1);

	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	virtual Element* isClicked(int x,int y);
	bool run();
	void unClick();
	int mouseGesture();

	int getValue(){return position;};
	static MySlider* slideRunning;
	int mySpeed;
};



//-----------------------------------------------------------------------------
// Section header
//-----------------------------------------------------------------------------

#define PADDING_SECT_H 5
#define SCROLL_WIDTH 5

class WINDLL  Section :
	public Element
{
protected:

	wstring title;			//titre de la section...
	static DIRECTDRAW_GDI *bkgSurface;
	RECT rcTitle;
	RECT clipping;


	//-----------------------------------------------------------------------------
	// user gesture :
	//-----------------------------------------------------------------------------
	int posOverlay;

	vector<Element*> components;

	Element* mouseAnalysePrev;
	Element* drawingHook;
	int prevPos;
	int prevOverlay;
	static int TITLE_SECTION_HEIGHT;
	static int TITLE_SECTION_WIDTH;

	bool canScroll;


	void printScrollBar(DIRECTDRAW_GDI* lpDDraw);
	void computeBkgSurface();
	//load (or reload) surface
	virtual void loadSurf();
public:
	//height is computed based on content...
	Section(int idElement, int x,int y,int width,wstring title);
	~Section(void);

	Element* selectedElement;
	static bool scrollVisible;
	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	virtual Element* isClicked(int x,int y);
	//Capture the mouse to interact with user
	virtual int mouseGesture();

	void add(Element* e,int minHeight=-1,int pos=-1,bool mixWithOther=false);
	void remove(unsigned int pos);

	void setDrawingHook(Element* e);

	inline void slideOverlay(int dY){posOverlay+=dY;};
	void updateScreen();
	void overlayVisibility(bool visib);

	void unselect() 
	{
		selectedElement=NULL;
	}
	void sizeChanged(Element* src,int w,int h);

	void scrollingAbility( bool canScroll ) 
	{
		this->canScroll=canScroll;
	}


	static GDI_OVERLAY *scrollBar;
};

//-----------------------------------------------------------------------------
// MyEditControl header
//-----------------------------------------------------------------------------

#define ID_EDITCHILD	102
#define IDM_EDITMENU	103

#define VIS_NORMAL 0
#define VIS_HOVER 1
#define VIS_DISABLED 2


using namespace std;

class WINDLL  MyEditControl :
	public Element
{
	static DIRECTDRAW_GDI *inputDDraw;
	static unsigned char widthCorner;//10...
	static unsigned short int widthItem;//450...

	MyLabel* text;
	bool slideToTop;
	DWORD startAnim;
	DWORD startPos;

	int state;//0=normal,1=hover, 2=disabled

	RECT clipping;
	Section* container;//the owner of the edit control...

	void show();
	//load (or reload) surface
	virtual void loadSurf();
public:
	MyEditControl(int idElement, int x,int y,int width);//no need for height (based on standard input height)
	~MyEditControl(void);

	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	virtual Element* isClicked(int x,int y);
	bool run();
	void unClick();
	int mouseGesture();

	wstring getText(){return text->getText();}
	void setText( LPCWSTR param1 );
	void sizeChanged(Element* src,int width,int height);
	static int EDIT_MENU_HEIGHT;

	static HWND input;//handle for text input...
	void showSIP();
	static MyEditControl* pEditCtrl;
};

//-----------------------------------------------------------------------------
// MyItem header
//-----------------------------------------------------------------------------

#define ITEM_IS_CHECKBOX		1
#define ITEM_IS_SWITCH			2
#define ITEM_IS_BUTTON_NORMAL	3
#define ITEM_IS_BUTTON_BIG		4
#define ITEM_IS_BUTTON_SHORT	5

#define ITEM_STATE_EBL_EMPTY		0
#define ITEM_STATE_EBL_SELECTED		1
#define ITEM_STATE_DSBL_EMPTY		2
#define ITEM_STATE_DSBL_SELECTED	3
#define ITEM_STATE_HOVER_EMPTY		4
#define ITEM_STATE_HOVER_SELECTED	5
#define ITEM_THIRD_STATE			6
#define ITEM_THIRD_STATE_DSBL		7

#define ITEM_COLOR_HOVER	0
#define ITEM_COLOR_DISABLED	1
#define ITEM_COLOR_NORMAL	2

class WINDLL  MyItem :
	public Element
{
	static DIRECTDRAW_GDI *checkBoxThrd;
	static DIRECTDRAW_GDI *switchThrd;
	static DIRECTDRAW_GDI *checkBoxes;
	static DIRECTDRAW_GDI *switches;
	static DIRECTDRAW_GDI *button_normal;
	static DIRECTDRAW_GDI *button_big;
	static DIRECTDRAW_GDI *button_short;

	static int checkBoxesWidth;
	static int checkBoxesHeight;
	static int switchesWidth;
	static int switchesHeight;
	static int button_normalWidth;
	static int button_normalHeight;
	static int button_bigWidth;
	static int button_bigHeight;
	static int button_shortWidth;
	static int button_shortHeight;
	static int button_ckThrdWidth;
	static int button_ckThrdHeight;
	static int button_swThrdWidth;
	static int button_swThrdHeight;

	DIRECTDRAW_GDI *mySurface;
	RECT clipping;
	RECT positionElement;
	RECT positionText;
	int elementHeight;
	int elementWidth;
	unsigned char state;
	unsigned char type;

	DWORD colorHover,colorDsbl,colorNrmal;
	DWORD colorBackHover,colorBackDsbl,colorBackNrmal;

	MyLabel* text;
	bool isCentered;
	//forcing to redraw
	void setMustRedraw();
	//load (or reload) surface
	virtual void loadSurf();
public:
	//no need for size (only depends of type)
	MyItem(int idElement, int x,int y,unsigned char type,unsigned char state,wstring text=L"");
	~MyItem(void);

	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	virtual Element* isClicked(int x,int y);
	bool run();
	void unClick();

	unsigned char isEnabled(){return state!=ITEM_STATE_DSBL_EMPTY&&state!=ITEM_STATE_DSBL_SELECTED&&state!=ITEM_THIRD_STATE_DSBL;};
	unsigned char isSelected(){return state==ITEM_STATE_HOVER_SELECTED||state==ITEM_STATE_EBL_SELECTED;};

	void changeText(wstring text);

	inline void switchVisibility(){
		if(state==ITEM_STATE_DSBL_EMPTY||state==ITEM_STATE_DSBL_SELECTED) state-=2;
		else if(state==ITEM_STATE_EBL_EMPTY||state==ITEM_STATE_EBL_SELECTED) state+=2;
		else if(state==ITEM_THIRD_STATE) state++;
		else if(state==ITEM_THIRD_STATE_DSBL) state--;
		else state-=2;
		setMustRedraw();
	}
	void sizeChanged(Element* src,int width,int height);
	void changeColorTxt(DWORD color,DWORD backColor,int type);
	void centerH();
};


//-----------------------------------------------------------------------------
// MyPanel header
//-----------------------------------------------------------------------------

//#define TITLE_PANEL_HEIGHT 32
#define PADDING_PANEL_H 0//was set to 5 but padding should be manually set...

class WINDLL  MyPanel :
	public Element
{
	wstring title;				//titre du panel...
	static DIRECTDRAW_GDI *bkgSurfaceSrc;
	DIRECTDRAW_GDI *bkgSurface;
	static int bkgSurfaceSrcWidth;
	static DIRECTDRAW_GDI *separator;
	static int separatorWidth;

	RECT rcTitle;		//size of title...
	RECT clipping;


	//////////////////////////////////////////////////////////////////////////
	//Configuration :
	//////////////////////////////////////////////////////////////////////////

	bool hasTitle,hasSeparator;

	vector<Element*> components;
	vector<int> heightComp;
	vector<bool> isAlone;

	void computeSurface();
	//load (or reload) surface
	virtual void loadSurf();
public:
	//height is computed based on content...
	MyPanel(int idElement, int x,int y,int width,wstring title,bool hasTitle=true,bool hasSeparator=true);
	~MyPanel(void);

	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	virtual Element* isClicked(int x,int y);
	void add(Element* e,int minHeight=-1,int pos=-1,bool mixWithOther=true);
	void remove(int pos);

	//////////////////////////////////////////////////////////////////////////
	// Getter :
	//////////////////////////////////////////////////////////////////////////

	Element* getElement(int pos){if((unsigned int)pos<components.size()&&pos>=0)return components[pos];else return NULL;};
	int getSize(){return components.size();};
	void sizeChanged(Element* src,int width,int height);
	static int TITLE_PANEL_HEIGHT;
};

//-----------------------------------------------------------------------------
// MySelector header
//-----------------------------------------------------------------------------


#define SELECT_STATE_NORMAL		0
#define SELECT_STATE_HOVER		1
#define SELECT_STATE_DSBL		2
#define SELECT_STATE_SELECTED	3

class WINDLL  MySelector :
	public Element
{
	static DIRECTDRAW_GDI *header;
	static int elementWidth;
	static int elementHeight;
	static int elementListWidth;
	static DIRECTDRAW_GDI *listMiddle;
	static DIRECTDRAW_GDI *listBottom;

	MyPanel *values;
	MyLabel* selected;
	RECT clipping;
	int state;

	Section* container;//the owner of the select control...
	MyLabel* title;
	bool animation;
	bool animToVisibility;
	bool hasPaintingHook;
	int lastY;
	int animIndex;
	bool isOpen;
	DWORD startSlide;
	//load (or reload) surface
	virtual void loadSurf();
public:
	//always the same size...
	MySelector(int idElement, int x,int y,wstring nameOfChoice);
	~MySelector(void);

	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	virtual Element* isClicked(int x,int y);
	bool run();
	void unClick();
	int mouseGesture();

	//////////////////////////////////////////////////////////////////////////
	// various behaviour :
	//////////////////////////////////////////////////////////////////////////

	void addItem(wstring word);
	int getSelection();
};


//-----------------------------------------------------------------------------
// ChildSection header
//-----------------------------------------------------------------------------

#define PADDING_ARROW 15

#define STATE_NORMAL	0
#define STATE_HOVER		1
#define STATE_DSBL		2
#define STATE_SELECTED	3

class WINDLL  ChildSection :
	public Element
{

	void slideScreen(DIRECTDRAW_GDI* lpDDraw);
protected:
	static DIRECTDRAW_GDI *backgroundLeft;//for speed, we copy arrow on background...
	static DIRECTDRAW_GDI *backgroundRight;

	static int arrowWidth;
	static int arrowHeight;
	static int backgroundHeight;
	static int backgroundWidth;

	Section* container;//the owner of the ChildSection...
	Section* ourSection;
	wstring savTitle;
	int savPaddingTopTitle;
	MyLabel* title;
	MyLabel* subTitle;


	double durationSlide;
	bool sliding;
	bool slidToLeft;
	bool isInOurSection;
	unsigned char state;
	RECT clipping;
	//load (or reload) surface
	virtual void loadSurf();
public:
	ChildSection(int idElement,int x,int y,wstring title,wstring subtitle=L"");
	~ChildSection(void);

	virtual void paint(DIRECTDRAW_GDI* lpDDraw,int x,int y);
	virtual Element* isClicked(int x,int y);

	bool run();
	void unClick();

	int mouseGesture();

	void add(Element *e,int minH=-1,int pos=-1,bool mixWithOther=false){ourSection->add(e,minH,pos,mixWithOther);};
	void remove(int pos){ourSection->remove(pos);};

	void changeSpeed(double newLength=200){durationSlide=newLength;};
};


//-----------------------------------------------------------------------------
// MessageBoxSense header
//-----------------------------------------------------------------------------


#define MESSAGE_ALERT		1
#define MESSAGE_YESNO		2
#define MESSAGE_OKCANCEL	3
#define MESSAGE_ABORTRETRY	4

#define MSG_REP_NO			0
#define MSG_REP_YES			1
#define MSG_REP_CANCEL		0
#define MSG_REP_OK			1
#define MSG_REP_ABORT		0
#define MSG_REP_RETRY		1

#define MESSAGE_REP			0x0500

class MessageBoxSense{
	MessageBoxSense(wstring title,wstring message,int type);

	static DIRECTDRAW_GDI *msgBox;
	static RECT rectBox;
	static int answer;
	static HANDLE threadMSG;

	static long FAR PASCAL messageDispatch(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	long messageAnalysis(UINT message, WPARAM wParam, LPARAM lParam);

	RECT posAbsolute;
	bool isInit;
	bool isActive;
	bool isAlreadyComputed;
	Element* selectedButton;
	int type;
	int heightMsg;
	int widthTitle;
	wstring title;
	wstring message;
	MyItem** button;
public:
	~MessageBoxSense();
	HWND hWnd;
	bool repaintButton;
	static int show(wstring title,wstring message,int type);
	void updateFrame(DIRECTDRAW_GDI* screen);
	//load (or reload) surface
	virtual void loadSurf();
};


//-----------------------------------------------------------------------------
// EventHandler header
//-----------------------------------------------------------------------------

#define EVENT_CLICK			1
#define EVENT_VALUE_CHANGED 2

class WINDLL  EventHandler
{
	HANDLE hThread;
	DWORD idThread;
	static DWORD WINAPI launchApp( LPVOID lpParameter );
protected:
	HINSTANCE gInstance;
	virtual void messageTreatment(Element* src,int idEvent)=0;
	virtual void initApplication()=0;

	void setApp(void* app);

public:
	EventHandler(HINSTANCE hInstance,wstring windowName,bool topmost=false);
	~EventHandler(void);

	void messageProc(Element* src,UINT msg=EVENT_CLICK);
	void runApp();
};

//-----------------------------------------------------------------------------
// DirectDrawWarp header
//-----------------------------------------------------------------------------

#define SW_MINIMIZED	6
#define MS_PER_FRAME	30
#define NB_FRAME_ANIM	1


class WINDLL  DirectDrawWarp
{
protected:

	//-----------------------------------------------------------------------------
	// data
	//-----------------------------------------------------------------------------
	LPDIRECTDRAW                g_pDD;				// DirectDraw object
	DIRECTDRAW_GDI*         g_pDDSPrimary;		// Primary Surface.

	DWORD previousFrame;
	bool isTopMost;

	// Our instance handle.


	const TCHAR* name;

	MouseCtrl* mouse;
	MessageBoxSense* messageBox;

	//-----------------------------------------------------------------------------
	// Window content
	//-----------------------------------------------------------------------------

	vector<Section*> section;
	int sectionActiv;
	int numRefreash;
	vector<RECT> bufferPaint;
	vector<bool> bufferPaintBack;
	//-----------------------------------------------------------------------------
	// Fonctions
	//-----------------------------------------------------------------------------

	// Desc: The Main Window Procedure
	static long FAR PASCAL StaticWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	long WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	int renderProc();

	int UpdateFramePriv(bool eraseBack=false);
	void doRepaintBuffered(int dy);

public:
	DirectDrawWarp(HINSTANCE hInstance,const TCHAR* name,bool topmost=false);
	~DirectDrawWarp(void);
	
	void UpdateFrame(bool eraseBack=false);
	void updateElements(){this->mouse->updateElement();};

	void add(Section* e);

	void changeControl(MessageBoxSense* mb);//when a message box append...

	void initWindow();
	void addToRedraw(RECT &rc,bool backErase);
	HRESULT BltFast(DIRECTDRAW_GDI* out,RECT *dest,DIRECTDRAW_GDI* srcSur, RECT *src,DWORD flag,LPDDBLTFX param);
	HRESULT BltMask(DIRECTDRAW_GDI* out,RECT *dest,DIRECTDRAW_GDI* srcSur,DIRECTDRAW_GDI* maskBlt, RECT *src);
	void BltMix(DIRECTDRAW_GDI* out,RECT *dest,DIRECTDRAW_GDI* srcSur, RECT *src,DWORD constantAlpha);
	static void deleteSurf(DIRECTDRAW_GDI** surf);
	static HRESULT createSurf(DDSURFACEDESC* desc,DIRECTDRAW_GDI* surf);
	static GDI_OVERLAY* createOverlay(DDSURFACEDESC* desc);
	static void beginPaint();
	static void endPaint();


	static int background_color;
};
